@@ -1,5 +1,5 @@
 /*
- * Copyright © 2007-2008 Gerd Kohlberger <lowfi@chello.at>
+ * Copyright © 2007-2009 Gerd Kohlberger <lowfi@chello.at>
  *
  * This file is part of Mousetweaks.
  *
@@ -18,7 +18,7 @@
  */
 
 /*
- * Modified version of dpid.c -libdaemon
+ * Modified version of dpid.c - libdaemon.
  * Copyright 2003-2007 Lennart Poettering <mzqnrzba (at) 0pointer (dot) de>
  */
 
@@ -38,6 +38,8 @@
 
 #include <glib.h>
 
+#include "mt-pidfile.h"
+
 #ifndef ETIME
 #define ETIME ETIMEDOUT /* For FreeBSD */
 #endif
